function [ x, lambda ] = Nullraum( A, b, B, g )
%Berechnet die Loesung des quadratischen Minimierungsproblems
%f(x)=1/2*x'*A*x+b'*x -> min unter der NB B*x=g
%mit dem Nullraumverfahren
%   Eingabe: Matrizen A,B, Vektoren b,g
%   Ausgabe: Loesung x, Lagrange-Multiplikator lambda

[m, n] = size(B);

%1. Schritt: Berechnung der Nullraum-Matrix 
%
%QR-Zerlegung von B^T  (F?r Q=[Y,Z] ist Z die Nullraum-Matrix)
[Q,R] = qr(B');
R = R(1:m,:);

%Berechnung von h und B fr Stabilitaet
h = -Q'*b;
B = Q'*A*Q;

%2. Schritt: Reduktion auf ein unrestringiertes Problem 
%
%Berechnung Y-Anteil der Loesung
xY = R'\g;
%Berechnung X-Anteil der Loesung
xZ = B(m+1:n,m+1:n)\(h(m+1:n)-B(m+1:n,1:m)*xY);
%Berechnung der Loesung
x = Q*[xY;xZ];

%3. Schritt
%Berechnung des Lagrange-Multiplikators
lambda = R\(h(1:m)-B(1:m,1:m)*xY-B(1:m,m+1:n)*xZ);

end